/* Patchwork database schema
 * version 0.1
 * 	for PostgreSQL databases
 */
begin;

CREATE SEQUENCE person_id_seq;
CREATE TABLE people (
	id		INTEGER NOT NULL DEFAULT nextval('person_id_seq'),
	email		TEXT NOT NULL,
	name		TEXT NULL,

	PRIMARY KEY (id),
	UNIQUE (email)
);
CREATE INDEX person_email_idx ON people USING BTREE (email);
GRANT SELECT ON people TO "www-data";
GRANT SELECT,INSERT,UPDATE ON people TO nobody;
GRANT SELECT,UPDATE ON person_id_seq TO nobody;

CREATE TABLE users (
	id		INTEGER NOT NULL,
	username	VARCHAR(64) NOT NULL,
	passwd		CHAR(32),

	PRIMARY KEY (id),
	FOREIGN KEY (id) REFERENCES people,
	UNIQUE(username)
);
GRANT SELECT ON users TO "www-data";

CREATE SEQUENCE state_id_seq;
CREATE TABLE states (
	id		INTEGER NOT NULL DEFAULT nextval('state_id_seq'),
	name		TEXT NOT NULL,
	ordering	INTEGER NOT NULL,

	PRIMARY KEY (id),
	UNIQUE (ordering)
);
GRANT SELECT ON states TO "www-data";

/*
 * Projects to which patches can be applied
 */
CREATE SEQUENCE project_id_seq;
CREATE TABLE projects (
	id		INTEGER NOT NULL DEFAULT nextval('project_id_seq'),
	name		TEXT NOT NULL,
	version		TEXT NOT NULL,
	alias		TEXT,

	PRIMARY KEY (id),
	UNIQUE (name, version)
);
GRANT SELECT ON projects TO "www-data";

/*
 * Bug report references
 */
CREATE SEQUENCE bugreport_id_seq;
CREATE TABLE bugreports (
	id		INTEGER NOT NULL DEFAULT nextval('bugreport_id_seq'),
	type		TEXT NOT NULL,
	ref		TEXT NOT NULL,

	PRIMARY KEY (id)
);
GRANT SELECT ON bugreports TO "www-data";
GRANT INSERT ON bugreports TO nobody;
GRANT SELECT,UPDATE ON bugreport_id_seq TO nobody;

/*
 * Patch submissions table
 */
CREATE SEQUENCE submission_id_seq;
CREATE TABLE submissions (
	id		INTEGER NOT NULL DEFAULT nextval('submission_id_seq'),
	msgid		TEXT NOT NULL,
	subject		TEXT NOT NULL,
	date		TIMESTAMP NOT NULL DEFAULT now(),
	submitter	INTEGER NOT NULL,
	npatches	INTEGER NOT NULL,

	PRIMARY KEY (id),
	FOREIGN KEY (submitter) REFERENCES people
);

CREATE INDEX submission_msgid_idx ON submissions USING BTREE (msgid);

GRANT SELECT,UPDATE,DELETE ON submissions TO "www-data";
GRANT SELECT,INSERT ON submissions TO nobody;
GRANT SELECT,UPDATE ON submission_id_seq TO nobody;

/*
 * Patch body table
 */
CREATE SEQUENCE patchfile_id_seq;
CREATE TABLE patchfiles (
	id		INTEGER NOT NULL DEFAULT nextval('patchfile_id_seq'),
	submission	INTEGER NOT NULL,
	patch		INTEGER NOT NULL,
	filename	TEXT,
	content		TEXT NOT NULL,

	PRIMARY KEY (id),
	FOREIGN KEY (submission) REFERENCES submissions ON DELETE CASCADE
);

CREATE INDEX patchfile_submission_idx ON patchfiles USING BTREE (submission);

GRANT SELECT,UPDATE,DELETE ON patchfiles TO "www-data";
GRANT SELECT,INSERT ON patchfiles TO nobody;
GRANT SELECT,UPDATE ON patchfile_id_seq TO nobody;

/*
 * Patch multiplexing table
 */
CREATE SEQUENCE patch_id_seq;
CREATE TABLE patches (
	id		INTEGER NOT NULL DEFAULT nextval('patch_id_seq'),
	state		INTEGER NOT NULL DEFAULT 1,
	project		INTEGER,
	bug		INTEGER,
	content		INTEGER NOT NULL,
	archived	BOOLEAN DEFAULT false,

	PRIMARY KEY (id),
	FOREIGN KEY (state) REFERENCES states,
	FOREIGN KEY (project) REFERENCES projects,
	FOREIGN KEY (bug) REFERENCES bugreports,
	FOREIGN KEY (content) REFERENCES patchfiles ON DELETE CASCADE
);

GRANT SELECT,UPDATE,DELETE ON patches TO "www-data";
GRANT SELECT,INSERT ON patches TO nobody;
GRANT SELECT,UPDATE ON patch_id_seq TO nobody;

/*
 * Comment table
 */
CREATE SEQUENCE comment_id_seq;
CREATE TABLE comments (
	id		INTEGER NOT NULL DEFAULT nextval('comment_id_seq'),
	submission	INTEGER NOT NULL,
	msgid		TEXT NOT NULL,
	submitter	INTEGER NOT NULL,
	date		TIMESTAMP NOT NULL DEFAULT now(),
	content		TEXT NOT NULL,

	PRIMARY KEY (id),
	FOREIGN KEY (submission) REFERENCES submissions ON DELETE CASCADE,
	FOREIGN KEY (submitter) REFERENCES people
);
CREATE INDEX comment_patch_idx ON comments USING BTREE (submission);
GRANT SELECT ON comments TO "www-data";
GRANT INSERT ON comments TO nobody;
GRANT SELECT,UPDATE ON comment_id_seq TO nobody;

CREATE SEQUENCE tree_id_seq;
CREATE TABLE trees (
	id		INTEGER NOT NULL DEFAULT nextval('tree_id_seq'),
	name		TEXT NOT NULL,
	url		TEXT,

	PRIMARY KEY (id)
);
GRANT SELECT ON trees TO "www-data";

CREATE TABLE patch_trees (
	patchfile	INTEGER NOT NULL,
	tree		INTEGER NOT NULL,
	date		TIMESTAMP NOT NULL DEFAULT now(),

	PRIMARY KEY (patchfile, tree),
	FOREIGN KEY (patchfile) REFERENCES patchfiles ON DELETE CASCADE,
	FOREIGN KEY (tree)  REFERENCES trees  ON DELETE CASCADE
);
GRANT SELECT,INSERT,DELETE ON patch_trees TO "www-data";

commit;
