# Patchwork - automated patch tracking system
# Copyright (C) 2005 Jeremy Kerr <jk@ozlabs.org>
#
# This file is part of the Patchwork package.
#
# Patchwork is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Patchwork is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Patchwork; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package PatchWork::SiteConfig;

use strict;
use Config::General;

my $configfile = '/etc/patchwork.conf';

# internal vars:
#  configfile	- the file to load config from
#  listfooter 	- text to remove from the bottom of each post
#  listid 	- ID of the list - appears in List-ID or X-Mailing-List header
#  dsn		- database source (as a DBI-style connection string)
#  dbuser	- database user
#  dbpass	- database password

sub new($@)
{
	my $cls = shift;
	my $obj = {};
	bless($obj, $cls);
	if (@_) {
		$obj->{configfile} = shift; 
	} else {
		$obj->{configfile} = $configfile;
	}
	$obj->{configtype} = $obj->_find_config_type(shift());
	return undef unless ($obj->load_config());
	return $obj;
}

sub _find_config_type($)
{
	my $obj = shift;
	if (@_) { return $_[0]; }
	return $ENV{PATCHWORK_CONFIG} if $ENV{PATCHWORK_CONFIG};
	
	my $name = 'patchwork-config';
	foreach my $file ("./.$name", $ENV{HOME}."/.$name",
				"/usr/local/share/patchwork/conf/$name") {
		open(FH, '<', $file) or next;
		my @lines = <FH>;
		close(FH);
		my $type = shift(@lines) or next;
		chomp $type;
		return $type if $type;
	}
}

sub load_config($)
{
	my ($obj) = @_;
	die "No config file defined" unless $obj->{configfile};
	my $conf = new Config::General(-ConfigFile => $obj->{configfile},
		-LowerCaseNames => 1);
	my %hash = $conf->getall() or
		die "Can't read config file ".$obj->{configfile};
	if ($obj->{configtype}) {
		return undef unless exists $hash{site}->{$obj->{configtype}};
		%hash = %{$hash{site}{$obj->{configtype}}};
	} else {
		my $default = each(%{$hash{site}});
		warn "No config type given, and there is no single default ".
			"(using $default)"
			if (scalar keys %{$hash{site}} > 1);
		%hash = %{$hash{site}->{$default}};
	}

	my %simplevars = (dsn => 'dsn', db_user => 'dbuser',
		db_pass => 'dbpass', 'list_footer' => 'listfooter',
		list_subject_regex => 'listsubjectregex', list_id => 'listid',
		web_include_dir => 'webincludedir', name => 'name',
		web_auth_type => 'webauthtype');

	$obj->{$_} = $hash{$simplevars{$_}} for (keys(%simplevars));

	return 1;
}

sub name($)
{
	my ($obj) = @_;
	return $obj->{name};
}

sub list_id($)
{
	my ($obj) = @_;
	return $obj->{list_id};
}

sub list_footer($)
{
	my ($obj) = @_;
	return $obj->{list_footer};
}

sub subject_regex($$)
{
	my ($obj) = @_;
	return $obj->{list_subject_regex};
}

sub dsn($)
{
	my ($obj) = @_;
	return $obj->{dsn};
}

sub db_auth($)
{
	my ($obj) = @_;
	return ($obj->{db_user}, $obj->{db_pass});
}

sub web_include_dir($)
{
	my ($obj) = @_;
	return $obj->{web_include_dir};
}

sub web_auth_type($)
{
	my ($obj) = @_;
	return $obj->{web_auth_type};
}

1;
